>module Types ( 
>  ProcessName, Argument, ProcessInfo, ProcessList, 
>  MsgNo, Player(Agent, Environment), AssignmentList, Test, 
>  ProtMsgDesc, ProtDesc, 
>  Secret(Sec, StrongSec), Secrets, Secrets',
>  AuthType(Aliveness, WeakAgreement, NonInjectiveAgreement, Agreement,
>           WeakIntensional, StrongIntensional,
>           TimedAliveness, TimedWeakAgreement, TimedNonInjectiveAgreement,
>           TimedAgreement, TimedWeakIntensional, TimedStrongIntensional),
>  Auth, Auths, AuthSigInfo, isTimedAuth, authArgs, 
>  FunctionList, Function(Symbolic,Defined, Inline), FunctionRHS, 
>  ActualAgent, SeqActualAgent(Star, SeqComp), ActualAgents, 
>  IntruderId, IntruderInitKnowledge, Deduction, Equivalence, CrackInfo, Cond,
>  Input,InputOLD, Inputpd,ExternalList,External, Output,Msg, VarSignalList, DiTypeList, IntruderProcsId,
>  FreshVarList, ChannelInfo,extern,notUsed,couple)

>where

>import Atoms
>import Messages




==========================================================================

>type ProcessName = String
>type Argument = String

>type ProcessInfo = (String, ProcessName, [Argument], [Msg],[VarName])
>type ProcessList = [ProcessInfo]

A ProcessInfo of the form

  ("a", "INITIATOR", ["a","na"], [Atom "PK", Apply "SK" (Atom "a")])

corresponds to a line in the input file of the form

  INITIATOR(a,na) knows PK, SK(a)

The first element of the tuple is the identity by which the agent is known in
the protocol description.

The second element is the name of the process that will be used to represent
the agent.

The third element is the list of variables that the agent initially knows, and
which will be instantiated with values in the #System section.

The fourth element of the tuple represents "knows" functions: functions that
the agent should be taken to know in the protocol description, but which will
not appear in the actual agent process.

=========================================================================
Protocol description

>type MsgNo = String

>data Player = Agent AgentId | Environment
>              deriving (Eq, Ord, Show)

>type AssignmentList = [(VarName, String)]

An element (v,e) represents that an agent should perform the assignment v:= e
before sending the next message.

>type Test = (String, Bool)

The first element gives the string representing the test; the second is a
boolean, saying whether the test uses the variable "now"

>type ProtMsgDesc = (AssignmentList, MsgNo, Player, Player, Msg, Test) 

>type ProtDesc = [ProtMsgDesc]

The element (asss, n, Agent A, Agent B, m, t) represents the protocol step

  (asss)
  n. A -> B : m
  [t]


========================================================================
Specifications

>data Secret = Sec AgentId Msg [AgentId] | StrongSec AgentId Msg [AgentId]
>              deriving (Eq, Show)
>type Secrets = [Secret]

>type Secret' = (AgentId, Msg, TypeName, [AgentId])
>type Secrets' = [Secret']

The specification `(a, v, bs)' means that a believes that v should remain
secret from all agents except those in as.

=========================================================================

Type of authentication specs

>data AuthType = Aliveness | 
>                WeakAgreement |
>                NonInjectiveAgreement [VarName] |
>                Agreement [VarName] | 
>                WeakIntensional |
>                StrongIntensional |
>                TimedAliveness Int |
>                TimedWeakAgreement Int |
>                TimedNonInjectiveAgreement Int [VarName] |
>                TimedAgreement Int [VarName] | 
>                TimedWeakIntensional Int |
>                TimedStrongIntensional Int
>                deriving Eq

>instance Show AuthType where
>  showsPrec _ Aliveness _ = "Aliveness"
>  showsPrec _ WeakAgreement _ = "WeakAgreement"
>  showsPrec _ (NonInjectiveAgreement ds) _ = 
>    "NonInjectiveAgreement"++showAuthArgs ds
>  showsPrec _ (Agreement ds) _ = "Agreement"++showAuthArgs ds
>  showsPrec _ WeakIntensional _ = "WeakIntensional"
>  showsPrec _ StrongIntensional _ = "StrongIntensional"
>  showsPrec _ (TimedAliveness t) _ = "TimedAliveness"++show t
>  showsPrec _ (TimedWeakAgreement t) _ = "TimedWeakAgreement"++show t
>  showsPrec _ (TimedNonInjectiveAgreement t ds) _ =
>    "TimedNonInjectiveAgreement"++show t++showAuthArgs ds
>  showsPrec _ (TimedAgreement t ds) _ = 
>    "TimedAgreement"++show t++showAuthArgs ds
>  showsPrec _ (TimedWeakIntensional t) _ = "TimedWeakIntensional"++show t
>  showsPrec _ (TimedStrongIntensional t) _ = "TimedStrongIntensional"++show t

>showAuthArgs = concat . map ('_' :)

=============================


>type Auth = (AgentId, AgentId, AuthType)
>type Auths = [Auth]
>type AuthSigInfo = (AgentId, AgentId, [VarName])

>isTimedAuth :: AuthType -> Bool
>isTimedAuth (TimedAliveness _) = True
>isTimedAuth (TimedWeakAgreement _) = True
>isTimedAuth (TimedNonInjectiveAgreement _ _) = True
>isTimedAuth (TimedAgreement _ _) = True
>isTimedAuth (TimedWeakIntensional _) = True
>isTimedAuth (TimedStrongIntensional _) = True
>isTimedAuth _ = False

>authArgs (NonInjectiveAgreement ds) = ds
>authArgs (Agreement ds) = ds
>authArgs (TimedNonInjectiveAgreement _ ds) = ds
>authArgs (TimedAgreement _ ds) = ds
>authArgs _ = []

An element of the form (B, A, at) means that B is correctly authenticated to
A, using authentication type at.


=========================================================================
Inline functions

>type FunctionRHS = VarName          -- only allow simple inline functions
>data Function = Symbolic VarName | 
>                Defined (VarName, [VarName], FunctionRHS) |
>                Inline (VarName, [VarName], String)
>              deriving (Eq, Show)
>type FunctionList = [Function]

>data External = Extern (String,String)
> deriving (Eq, Show)
>type ExternalList = [External]

>extern=Extern

>couple::ExternalList -> [(String,String)]
>couple [] = []
>couple ((Extern(a,b)):q) = ((a,b):(couple q))

>-- isSymbolic :: InlineList -> VarName -> Bool
>-- isSymbolic inlines f = or [f==f' | Symbolic f' <- inlines]

=========================================================================
Actual players in system

>type ActualAgent = (ProcessName, [Argument])

("INITIATOR", ["Alice", "Na", "PKa", "SKa", "Sam", "PKs"]) represents the
agent modelled by the process INITIATOR(Alice, Na, PKa, SKa, Sam, PKs).

>data SeqActualAgent = Star ActualAgent | SeqComp [ActualAgent]
>                      deriving Show

>type ActualAgents = [SeqActualAgent]
>type WithdrawOption = Bool
>type StaleKnowledge = Bool

=========================================================================
Name and initial knowledge of intruder; type of deductions

>type IntruderInitKnowledge = [Msg] 
>type IntruderId = String
>type IntruderProcesses = [ProcessName]

>type Deduction = (VarTypeList, [Msg], Msg)
>type Cond = (VarName, TypeName)

========================================================================
Equivalences

>type Equivalence = (VarTypeList, Msg, Msg)

========================================================================
Channel information

>type ChannelInfo = (Bool, Bool, Bool)

Values in order specify whether channels should be secret, authenticated, and
whether there should be a direct channel between honest participants.

Remember: SAD (secret, authenticated, direct)

========================================================================
Crackable information

>type CrackInfo = (TypeName, Maybe Int)

=========================================================================
Complete input

>type Input = 
>  (VarTypeList, FnTypeList, InverseKeyList, [DataTypeDef], 
>   ProcessList, ProtDesc, --Secrets, Auths, ActVarTypeList,
>   --InverseKeyList, TimeStampInfo, 
>   FunctionList,ExternalList)--, ActualAgents,  
>   --WithdrawOption, StaleKnowledge, IntruderId, IntruderInitKnowledge, 
>   --IntruderProcesses, [CrackInfo], [Deduction], [TypeName], 
>   --[Equivalence], ChannelInfo)

>type InputOLD = 
>  (VarTypeList, FnTypeList, InverseKeyList, [DataTypeDef], 
>   ProcessList, ProtDesc, Secrets, Auths, ActVarTypeList,
>   InverseKeyList, TimeStampInfo, 
>   FunctionList, ActualAgents,  
>   WithdrawOption, StaleKnowledge, IntruderId, IntruderInitKnowledge, 
>   IntruderProcesses, [CrackInfo], [Deduction], [TypeName], 
>   [Equivalence], ChannelInfo)

>type Inputpd = 
>  (VarTypeList, FnTypeList, InverseKeyList, [DataTypeDef], 
>   ProcessList, ProtDesc, Secrets, Auths)


>type Output = String

The components are:

VarTypeList: The list of free variables used in the protocol
description, with their types;

FnTypeList: List of function names with their types;

InverseKeyList: The list of key inverses for keys used in the protocol
description;

ProcessList: The list of process names and arguments for agents
appearing in the protocol description;

ProtDesc: The protocol description itself;

Secrets: The secret specifications that the protocol is supposed to
meet;

Auths: The authentication specifications that the protocol is supposed
to meet;

VarTypeList: The list of actual variables;

InverseKeyList: The list of inverses for actual keys;

TimeStampInfo: Information about how time is used in the system

InlineList: A list of inline functions;

ActualAgents: The actual agents taking part in the system;

IntruderId: The identity of the intruder

IntruderInitKnowledge: The initial knowledge of the intruder;

[Deduction]: The list of extra deductions.

guessables :: [TypeName] list of guessable types

ChannelInfo :: info about whether channels are secret or authenticated

======================================================================
Further types declared:

(For signal events)

>type VarSignalList = [(MsgNo,AgentId,[VarName])]

used for storing the variables required for some spec. by a given
agent in a given message.

(For Data Independence)

>type DiTypeList = [TypeName]  -- dit

>type IntruderProcsId = [AgentId]  -- intruderProcsId

>type FreshVarList = [(MsgNo,(AgentId,[VarName]),(AgentId,[VarName]))]
> -- accumList
