package protocols;

import core.*;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.lang.reflect.Array;
import java.io.ObjectOutputStream;
import java.io.ByteArrayOutputStream;
import java.math.BigInteger;

public class BANK extends Protocol {
   
   public BANK (ObjectOutputStream oos) {
      super("core.MessageT","core.MessageKeyT","core.NonceT");
      this.oos=oos;
      }
   
   public void run() {
      try {
         
         println("creating nonce transactionID");
         createNewNonce("transactionID");
         println("creating nonce summary");
         createNewNonce("summary");
         
         println("waiting for m4");
         Message m4 = message(get("C",4000));
         checkTag(m4,"APASA)N)");
         addParts(m4.getAllFields(),2);
         m0=getPart();
         setNameToId(m0.getString(),"C");
         m0=getPart();
         println("Decrypting m0");
         m0.decrypt("RSA", getPrivateKey(nameFromId("B")));
         checkTag(m0,"ASA)N");
         addParts(m0.getAllFields(),3);
         m0=getPart();
         setNameToId(m0.getString(),"A");
         m0=getPart();
         println("Decrypting m0");
         m0.decrypt("RSA", getPublicKey(nameFromId("C")));
         checkTag(m0,"A");
         addParts(m0.getAllFields(),1);
         m0=getPart();
         setNameToId(m0.getString(),"B");
         
         m0=getPart();
         setValueToNonce(m0.getString(),"price");
         
         
         
         println("Building message m5");
         Message m5 =message();
         m5.addMessage(message(nonceFromId("price")));
         m5.addMessage(message(nonceFromId("transactionID")));
         println("Encrypting m5");
         m5.encrypt("RSA", getPublicKey(nameFromId("C")));
         println("Sending message m5");
         put("C", m5.getString());
         
         println("waiting for m6");
         Message m6 = message(get("C"));
         checkTag(m6,"PNSPN)))");
         println("Decrypting m6");
         m6.decrypt("RSA", getPrivateKey(nameFromId("B")));
         checkTag(m6,"NSPN))");
         addParts(m6.getAllFields(),2);
         m0=getPart();
         setValueToNonce(m0.getString(),"transactionID");
         m0=getPart();
         println("Decrypting m0");
         m0.decrypt("RSA", getPublicKey(nameFromId("A")));
         checkTag(m0,"PN)");
         addParts(m0.getAllFields(),1);
         m0=getPart();
         println("Decrypting m0");
         m0.decrypt("RSA", getPrivateKey(nameFromId("B")));
         checkTag(m0,"N");
         addParts(m0.getAllFields(),1);
         m0=getPart();
         setValueToNonce(m0.getString(),"carddata");
         
         
         
         
         println("Building message m7");
         Message m7 =message();
         m0=message();
         m0.addMessage(message(agentFromId("C")));
         println("Encrypting m0");
         m0.encrypt("RSA", getPrivateKey(nameFromId("B")));
         m7.addMessage(m0);
         m0=message();
         m0.addMessage(message(nonceFromId("carddata")));
         println("Encrypting m0");
         m0.encrypt("RSA", getPublicKey(nameFromId("A")));
         println("Encrypting m0");
         m0.encrypt("RSA", getPrivateKey(nameFromId("B")));
         m7.addMessage(m0);
         m7.addMessage(message(nonceFromId("summary")));
         println("Encrypting m7");
         m7.encrypt("RSA", getPublicKey(nameFromId("C")));
         println("Sending message m7");
         put("C", m7.getString());
         
         println("waiting for m10");
         Message m10 = message(get("C"));
         checkTag(m10,"PSA)NN)");
         println("Decrypting m10");
         m10.decrypt("RSA", getPrivateKey(nameFromId("B")));
         checkTag(m10,"SA)NN");
         addParts(m10.getAllFields(),3);
         m0=getPart();
         println("Decrypting m0");
         m0.decrypt("RSA", getPublicKey(nameFromId("C")));
         checkTag(m0,"A");
         addParts(m0.getAllFields(),1);
         m0=getPart();
         setNameToId(m0.getString(),"B");
         
         m0=getPart();
         setValueToNonce(m0.getString(),"acknowledgement");
         m0=getPart();
         setValueToNonce(m0.getString(),"end");
         
         
         writeObject(nameFromId("A"));
         writeObject(nameFromId("C"));
         writeObject(nonceFromId("price"));
         writeObject(nonceFromId("summary"));
         
         
         } catch (Exception e) {e.printStackTrace();System.exit(0);}
      }
   
   public static void subMain(String[] args,ObjectOutputStream oos) {
      BANK B;
      while (true) {
         B=new BANK(oos);
         args=B.getArgs(args);
         B.newAgent("C",4000);
         B.start();
         }
      }
   
   public String[] getArgs(String[] args) {
      try {
         if (args.length != 3) {
            args = (String[]) Array.newInstance("".getClass(), 3);
            BufferedReader in2 =
            new BufferedReader(new InputStreamReader(System.in));
            println("Type the KeyStore file to use  [keystore]");
            args[0]=in2.readLine();
            if (args[0].compareTo("")==0) args[0]="keystore";
            println("Type the password to access it [duitama]");
            args[1]=in2.readLine();
            if (args[1].compareTo("")==0) args[1]="duitama";
            println("Type the name of agent B [B]");
            args[2]=in2.readLine();
            if (args[2].compareTo("")==0) args[2]="B";
            }
         setKS(args[0],args[1]);
         setNameToId(args[2],"B");
         } catch (Exception e) {e.printStackTrace();}
      return args;
      }
   
   public static void main(String[] args) {
      ObjectOutputStream oos=null;
      try {
         ByteArrayOutputStream os = new ByteArrayOutputStream();
         oos=new ObjectOutputStream(os);
         PrintOOS p=new PrintOOS("BANK",oos,os);
         p.start();}
      catch (Exception e) {e.printStackTrace();}
      subMain(args,oos);
      }
   }