package protocols;

import core.*;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.lang.reflect.Array;
import java.io.ObjectOutputStream;
import java.io.ByteArrayOutputStream;
import java.math.BigInteger;

public class RESPXAV extends Protocol {
   
   public RESPXAV (ObjectOutputStream oos) {
      super("core.Message" ,"core.MessageKey" ,"core.Nonce" );
      this.oos=oos;
      }
   
   public void run() {
      try {
         
         
         println("waiting for m1");
         Message m1 = message(get("A",4000));
         addParts(m1.getAllFields(),2);
         m0=getPart();
         println("Decrypting m0");
         m0.decrypt("RSA", getPrivateKey(nameFromId("B")));
         addParts(m0.getAllFields(),3);
         m0=getPart();
         setNameToId(m0.getString(),"A");
         m0=getPart();
         setNameToId(m0.getString(),"B");
         m0=getPart();
         setValueToNonce(m0.getString(),"na");
         
         m0=getPart();
         println("Decrypting m0");
         m0.decrypt("RSA", getPublicKey(nameFromId("A")));
         addParts(m0.getAllFields(),1);
         m0=getPart();
         /* Hash matching*/
         {println("Building message mhash");
            storeM0();
            Message mhash = message();
            mhash.addMessage(message(agentFromId("A")));
            mhash.addMessage(message(agentFromId("B")));
            mhash.addMessage(message(nonceFromId("na")));
            println("Encrypting mhash");
            mhash.encrypt("RSA", getPublicKey(nameFromId("B")));
            m0=restoreM0();
            mhash.hash("MD5");
            if (m0.equals(mhash)) println("hash is correct !");
            else throw new Exception("Incorrect hash in message m0!"+m0.getString()+" "+mhash.getString());
            }
         
         
         println("Building message m2");
         Message m2 =message();
         m2.addMessage(message(agentFromId("A")));
         m2.addMessage(message(agentFromId("B")));
         m2.addMessage(message(nonceFromId("na")));
         println("Encrypting m2");
         m2.encrypt("RSA", getPublicKey(nameFromId("A")));
         println("Sending message m2");
         put("A", m2.getString());
         
         writeObject(nameFromId("A"));
         writeObject(nonceFromId("na"));
         
         
         } catch (Exception e) {e.printStackTrace();System.exit(0);}
      }
   
   public static void subMain(String[] args,ObjectOutputStream oos) {
      RESPXAV B;
      while (true) {
         B=new RESPXAV(oos);
         args=B.getArgs(args);
         B.newAgent("A",4000);
         B.start();
         }
      }
   
   public String[] getArgs(String[] args) {
      try {
         if (args.length != 3) {
            args = (String[]) Array.newInstance("".getClass(), 3);
            BufferedReader in2 =
            new BufferedReader(new InputStreamReader(System.in));
            println("Type the KeyStore file to use  [keystore]");
            args[0]=in2.readLine();
            if (args[0].compareTo("")==0) args[0]="keystore";
            println("Type the password to access it [duitama]");
            args[1]=in2.readLine();
            if (args[1].compareTo("")==0) args[1]="duitama";
            println("Type the name of agent B [B]");
            args[2]=in2.readLine();
            if (args[2].compareTo("")==0) args[2]="B";
            }
         setKS(args[0],args[1]);
         setNameToId(args[2],"B");
         } catch (Exception e) {e.printStackTrace();}
      return args;
      }
   
   public static void main(String[] args) {
      ObjectOutputStream oos=null;
      try {
         ByteArrayOutputStream os = new ByteArrayOutputStream();
         oos=new ObjectOutputStream(os);
         PrintOOS p=new PrintOOS("RESPXAV",oos,os);
         p.start();}
      catch (Exception e) {e.printStackTrace();}
      subMain(args,oos);
      }
   }